#!/usr/bin/python
#^#
# OVSAsyncTaskSupport.py
# Asynchronous task Support functions
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
# hongxi.ma@oracle.com 2008.1.20
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import logging
from OVSServerProxy import ServerProxy
from OVSCommons import exposed, fmt_err_msg, run_cmd
from OVSAsyncTaskAux import get_async_auxdata, save_async_auxdata3
from os.path import exists, basename, join
from os import stat,system

@exposed
def wget_status(taskid):
    logger = logging.getLogger("ovs.query")
    try:
        agt_url = get_async_auxdata(taskid, 'utility_agent')
        if agt_url is None:
            raise Exception('can not get utility_agent url')
        sp = ServerProxy(agt_url)
        logger.info('wget_status: the utility server for task %s => %s' % (taskid, sp.agt_url))
        rs = sp.utl_wget_status(taskid)
        logger.info('wget_status: success. %s => %s' % (taskid, rs))
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('wget_status: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)
@exposed
def wget_cancel(taskid):
    logger = logging.getLogger("ovs.operation")
    try:
        agt_url = get_async_auxdata(taskid, 'utility_agent')
        if agt_url is None:
            raise Exception('can not get utility_agent url')
        sp = ServerProxy(agt_url)
        logger.info('wget_cancel: success. %s => %s' % (taskid, sp.agt_url))
        rs = sp.remote_async_task_kill(taskid)
        if not rs.startswith('success'):
            raise Exception('remote_async_task_kill faield with message: %s' % rs)
        try:
            rs = utl_wget_cancel(taskid)
        except:
            pass
        return rs
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('wget_cancel: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)

@exposed
def utl_wget_status(taskid):
    logger = logging.getLogger("ovs.query")
    try:
        logfile = get_async_auxdata(taskid, 'wgetlog')
        target_dir = get_async_auxdata(taskid, 'target_dir')
        if not logfile:
            raise Exception('can not get wget logfile')
        if not exists(logfile):
            raise Exception('logfile doesnot exists: %s' % logfile)
        protocol, fixed_protocol = get_protocol(taskid)
        fi = get_downloading_fileinfo(logfile, protocol, target_dir)
        if not fi and not fixed_protocol:
            if protocol == 'http':
                protocol = 'ftp'
            else:
                protocol = 'http'
            fi = get_downloading_fileinfo(logfile, protocol, target_dir)
        if not fi:
            logger.warning('utl_wget_status: success. %s => %s' % (taskid, 'get no downloading file information.'))
            return 'success:'
        else:
            if not fixed_protocol:
                try:
                    save_async_auxdata3(taskid, {'protocol': protocol})
                except:
                    pass
            logger.info('utl_wget_status: success. %s => %s,%s,%s' % ((taskid,) + fi))
            return 'success:%s,%s,%s' % fi
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('utl_wget_status: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)

def get_protocol(taskid):
    protocol = get_async_auxdata(taskid, 'protocol')
    fixed_protocol = bool(protocol)
    if not fixed_protocol:
        url = get_async_auxdata(taskid, 'url')
        if not url:
            raise Exception('can not get wget url')
        url = url.lower()
        if url.startswith('http'):
            protocol='http'
        elif url.startswith('ftp'):
            protocol='ftp'
        else:
            raise Exception('Unkown protocol for url: %s' % url)
    return (protocol, fixed_protocol)

def get_downloading_fileinfo(logfile, protocol, target_dir):
    targets = get_wget_targets(logfile, protocol, target_dir)
    if not targets:
        return None

    targets.reverse()
    target,length = targets[0]
    if not length:
        length = '-1'

    fn = basename(target)
    if not exists(target):
        fs = '0'
    else:
        fi = stat(target)
        fs = str(fi[6])
    return (fn, length, fs)

def get_wget_targets(logfile, protocol, target_dir):
    if protocol == 'http':
        targets = get_wget_targets_http(logfile)
    elif protocol == 'ftp':
        targets = get_wget_targets_ftp(logfile)
    else:
        raise Exception('Unkown protocol %s for get_wget_targets.' % protocol)

    result = []
    for filename, length in targets:
        if not filename.startswith('/'):
            filename = join(target_dir, filename)
        if exists(filename):
            result.append((filename, length))
    return result

def get_wget_targets_http(logfile):
    target_prefix="Saving to:"
    def get_target_file(line):
        if line.startswith(target_prefix):
            target = line.replace(target_prefix, '', 1).strip()
            if target.startswith("`") and target.endswith("'"):
                return target[1:-1]

    def get_target_length(line):
        if line.startswith('Length:'):
            segs = line.split(' ')
            if len(segs) > 1:
                length = segs[1].strip()
                try:
                    if long(length) > 0:
                        return length
                except:
                    pass

    targets = []
    try:
        lines = run_cmd("cat '%s' | grep -B 1 '^%s' | grep -v '^\-\-$'" % (logfile, target_prefix))
    except:
        pass
    else:
        length = None
        for line in lines.splitlines():
            if line == '':
                continue
            if line.startswith('Length:'):
                length = get_target_length(line)
            elif line.startswith(target_prefix):
                target = get_target_file(line)
                if target:
                    targets.append((target, length))
                length = None
    return targets

def get_wget_targets_ftp(logfile):
    target_prefix="           =>"
    def get_target_file(line):
        if line.startswith(target_prefix):
            target = line.replace(target_prefix, '', 1).strip()
            if target.startswith("`") and target.endswith("'"):
                return target[1:-1]

    def get_target_length(line):
        if line.startswith('Length:'):
            segs = line.split(' ')
            if len(segs) > 1:
                length = segs[1].strip()
                try:
                    if long(length) > 0:
                        return length
                except:
                    pass

    targets = []
    try:
        lines = run_cmd("cat '%s' | grep -E  '^%s|^Length:'" % (logfile, target_prefix))
    except:
        pass
    else:
        length = None
        target = None
        for line in lines.splitlines():
            if line == '':
                continue
            if line.startswith('Length:'):
                length = get_target_length(line)
            elif line.startswith(target_prefix):
                if target:
                    targets.append((target, length))
                length = None
                target = get_target_file(line)
        if target and exists(target):
            targets.append((target, length))
    return targets

@exposed
def utl_wget_cancel(taskid):
    logger = logging.getLogger("ovs.operation")
    try:
        logfile = get_async_auxdata(taskid, 'wgetlog')
        target_dir = get_async_auxdata(taskid, 'target_dir')
        if not logfile:
            raise Exception('can not get wget logfile')
        if not exists(logfile):
            raise Exception('logfile doesnot exists: %s' % logfile)
        protocol, fixed_protocol = get_protocol(taskid)
        #1. kill wget
        targets = get_wget_targets(logfile, protocol, target_dir)
        if not targets and not fixed_protocol:
            if protocol == 'http':
                protocol = 'ftp'
            else:
                protocol = 'http'
            targets = get_wget_targets(logfile, protocol, target_dir)
            
        system("fuser -k '%s' &> /dev/null" % logfile)
        #2. remove downloaded data
        for target,length in targets:
            if target:
                try:
                    system("rm -rf '%s'" % target)
                except:
                    pass
        #3. remove log file, not necessary actully, since fuser -k will remove it.
        system("rm -rf '%s'" % logfile)
            
        logger.info('utl_wget_cancel: success. %s => %s' % (taskid, repr(targets)))
        return 'success'
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('utl_wget_cancel: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)

@exposed
def create_vm_disk_status(taskid):
    logger = logging.getLogger("ovs.query")
    try:
        disk_path = get_async_auxdata(taskid, 'disk_path')
        required_size = get_async_auxdata(taskid, 'size')
        if not disk_path:
            raise Exception('can not get disk_path')
        if not required_size:
            raise Exception('can not get required disk size')
        disk_name = basename(disk_path)
        if not exists(disk_path):
            created_size = '0'
        else:
            fi = stat(disk_path)
            created_size = str(fi[6])

        logger.info('create_vm_disk_status: success. %s => %s,%s,%s' % (taskid, disk_name, required_size, created_size))
        return 'success:%s,%s,%s' % (disk_name, required_size, created_size)
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('create_vm_disk_status: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)

@exposed
def create_vm_disk_cancel(taskid):
    logger = logging.getLogger("ovs.operation")
    try:
        disk_path = get_async_auxdata(taskid, 'disk_path')
        if not disk_path:
            raise Exception('can not get disk_path')
        if exists(disk_path):
            system("fuser -k '%s' &> /dev/null" % disk_path)
            system("rm -rf '%s'" % disk_path)
            logger.info('create_vm_disk_cancel: success. %s => %s' % (taskid, disk_path))
        else:
            logger.warning('create_vm_disk_cancel: %s => disk_path: %s not exist.' % (taskid, disk_path))
        return 'success'
    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error('create_vm_disk_cancel: failed! %s => %s' % (taskid, errmsg))
        return 'failed:%s' % (errmsg,)
