#!/usr/bin/env python

#^#
# OVSConfig.py
#
# <description>
#
# 
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

from ConfigParser import ConfigParser
from os import makedirs, remove
from os.path import dirname, join, isfile, isdir, exists
from OVSConstants import CONFIG_DIR
from shutil import copyfile
from signal import signal, SIGUSR1

_g_config = None
_g_passwd = None

def init_config():
    #make default dirs
    dbdir = join(CONFIG_DIR, "db")
    if not isdir(dbdir): makedirs(dbdir)
    certdir = join(CONFIG_DIR, "cert")
    if not isdir(certdir): makedirs(certdir)

    #agent configuration file
    filename = join(CONFIG_DIR, "agent.ini")
    filename_default = join(dirname(__file__), "conf/agent.ini.default")
    if not isfile(filename):
        if isfile(filename_default):
            copyfile(filename_default, filename)
        else:
            raise Exception("init_config: config file '%s' does not exist." % filename)

    global _g_config 
    _g_config = ConfigParser()
    _g_config.read(filename)

    #init default passwdfile
    try:
        filename = _g_config.get("security", "passwdfile")
    except:
        filename = join(CONFIG_DIR, "passwdfile")
    filename_default = join(dirname(__file__), "conf/passwdfile.default")
    if not isfile(filename):
        if isfile(filename_default):
            copyfile(filename_default, filename)
        else:
            raise Exception("init_config: passwdfile file '%s' does not exist." % filename)

def init_passwd():
    global _g_passwd
    _g_passwd = {}
    config = get_config()

    l = []
    try:
        filename = config.get("security", "passwdfile")
    except:
        filename = join(CONFIG_DIR, "passwdfile")
    if exists(filename):
        l.append(filename)
    try:
       if config.get("ha", "remaster").lower().startswith("enable"):
            filename = config.get("ha", "passwdfile.shared")
            if exists(filename):
                l.append(filename)
    except:
        pass
    if not l:
        raise Exception("init_passwd: No password file found.")

    for filename in l:
        try:
            f = open(filename, "r")
            try:
                lines = f.read().splitlines()
                for line in lines:
                    username, passwd_digest = line.strip().split(":{SHA}")
                    _g_passwd[username] = passwd_digest
            finally:
                f.close()
        except:
            continue

    if not _g_passwd:
        raise Exception("init_passwd: No password found from password file: %s" % repr(l))

def get_config():
    if not _g_config: 
        init_config()
    return _g_config

def get_passwd():
    if not _g_passwd:
        init_passwd()
    return _g_passwd

def clear_cache():
    global _g_config
    global _g_passwd
    _g_config = None
    _g_passwd = None

def sig_func(signum, frame):
    if signum != SIGUSR1:
        return
    clear_cache()
    dirty = join(CONFIG_DIR, "dirtycache")
    if exists(dirty):
        try:
            remove(dirty)
        except:
            pass

def init_signal_handler():
    signal(SIGUSR1, sig_func)
