#!/usr/bin/env python
#^#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import re
import os
import logging
from time import sleep
from os.path import exists, join, dirname
from OVSLog import init_log
from OVSDaemon import daemonize
from OVSConfig import init_config, get_config
from OVSCommons import run_cmd, fmt_err_msg
from OVSCDS import CDS

#source in OVSPolicy
execfile(join(dirname(__file__), "OVSPolicy.py"))
#

def execute_policy():
    logger = logging.getLogger()
    try:
        #only execute policy when pool ha enable
        pool_ha_enable = False
        cds = CDS("master")
        try:
            if "pool_ha_enable" in cds.db_master:
                pool_ha_enable = cds.db_master["pool_ha_enable"]
        finally:
            cds.close()
        if not pool_ha_enable: return

        #execute policy apply to vm
        cds = CDS("vm")
        vms = {}
        try:
            vms.update(cds.db_vm)
        finally:
            cds.close()

        for vm_path, vm in vms.items():
            if not "policy" in vm:
                continue
            l = vm["policy"] #list of (priority, policy_name)
            #policies sorted by priority
            l.sort()
            l.reverse()
            for priority, policy_name in l:
                if policy_name not in policy:
                    continue
                trigger, action, action_params, on_success, on_error = policy[policy_name]
                try:
                    if not eval(trigger):
                        continue
                except KeyError: #key may not exist in the metrics dictionary
                    continue

                #execute the policy
                logger.info("OVSPolicyServer.execute_policy(): vm('%s'), action=%s action_params=%s => start..." % \
                        (vm_path, action, action_params))
                rs = eval(action)(*eval(action_params))
                logger.info("OVSPolicyServer.execute_policy(): vm('%s'), action=%s action_params=%s => %s" % \
                        (vm_path, action, action_params, rs))

                if rs.startswith("success"):
                    if not on_success or on_success == "EXIT":
                        break
                    if on_success == "NEXT":
                        continue
                else:
                    if not on_error or on_error == "EXIT":
                        break
                    if on_error == "NEXT":
                        continue

        #TODO:execute policies applied to server?

    except Exception, e:
        errmsg = fmt_err_msg(e)
        logger.error("OVSPolicyServer.execute_policy(): error. => %s" % errmsg)

def run():
    logger = logging.getLogger()
    while True:
        try:
            init_config() #the change will take effect immediately
            config = get_config()
            if config.get("ha", "policy.execute").startswith("enable"):
                execute_policy()
        except Exception, e:
            errmsg = fmt_err_msg(e)
            logger.error("OVSPolicyServer.run(): error. => %s" % errmsg)

        #next cycle ...
        try:
            config = get_config()
            interval = int(config.get("ha", "policy.execute.interval"))
            sleep(interval)
        except:
            sleep(60)

def main():
    from os.path import basename
    pid_filename = basename(__file__).split(".")[0]
    daemonize(pid_filename)

    init_log("client")
    run()

if __name__ == "__main__":
    main()

