#/usr/bin/env python
# Copyright (C) 2008, Oracle
# clone.py
# OracleVM 2.1.2
# Clone vm
import os
import signal
from subprocess import Popen
from subprocess import PIPE
import threading
import fcntl
import select

debug = 0

_clone_sessions = {}
_clone_sessions_status = {}

def _session_status_task(pid):
    global debug
    while 1:
        outfd = _clone_sessions[pid].stdout
        ready = select.select([outfd], [], [])
        if outfd in ready[0]:
            outcheck = outfd.read()
            if outcheck == '':
                _clone_sessions_status[pid] = "Done or canceled"
                break
            _clone_sessions_status[pid] = outcheck
            if debug > 1:
                print 'outcheck:', outcheck
            if outcheck.strip() == "Done":
                break

# Start a clone session
def clone_start(cfg_file_path, target_name):
    global debug
    try:
        p = Popen(["./utils/oraclevm-clone.sh", cfg_file_path, target_name], stdout=PIPE)
        _clone_sessions[p.pid] = p

        # nonblock read
        fl = fcntl.fcntl(p.stdout, fcntl.F_GETFL)
        fcntl.fcntl(p.stdout, fcntl.F_SETFL, fl | os.O_NDELAY)

        t = threading.Thread(target = _session_status_task, args = (p.pid,))
        t.start()

        # create a daemon
        try:
            pid = os.fork()
        except OSError, e:
            raise Exception, "%s [%d]" % (e.strerror, e.errno)
        if pid == 0: # child
            os.setsid()
            try:
                pid = os.fork()
            except OSError, e:
                raise Exception, "%s [%d]" % (e.strerror, e.errno)
            if pid == 0: # second child
                os.chdir("/")
                os.umask(0)
                t.join()
            else:
                os._exit(0) # Exit first child
        else:
            return "sucess:%d" % p.pid

    except Exception, e:
        if debug:
            print e
        return "failed:", e

# Get the status of cones sessions
# If the session_id is null, return the a list containing all the sessions
# Return:
#  success: "success:%d,%s;..."%(session_id, status)...
#  failed:  "failed:%s"%error_message
def clone_status(session_id):
    if not session_id:
        str = ''
        for k,v in _clone_sessions_status.iteritems():
            str += "%d,%s" % (k, v)
        return "sucess:%s" % str

    if _clone_sessions_status.has_key(session_id):
        return "sucess:%s", _clone_sessions_status[session_id]

    return "failed:session not found"

# Cancel a clone session
def clone_cancel(session_id):
    if _clone_sessions.has_key(session_id):
        try:
            os.kill(session_id, signal.SIGTERM)
        except OSError,e:
            return "failed:", e
    else:
        return "failed:session not found"

if __name__ == "__main__":
    debug = 1
    ret = clone_start("/root/clonetest/ttylinuxvm/ttylinux_vm", "ttylinuxclone1")
    print "ret=:", ret

    import time
    while 1:
        print clone_status(None)
        time.sleep(2)
