#!/usr/bin/env python
#^#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import sys
from subprocess import Popen, PIPE
from os.path import dirname, join
from getopt import getopt, GetoptError

def usage():
    print >>sys.stderr, "%s --lock --name <name> --uuid <uuid>"
    print >>sys.stderr, "%s --unlock --name <name> --uuid <uuid>"

if __name__ == "__main__":
    try:
        opts, args = getopt(sys.argv[1:], "", ["lock", "unlock", "name=", "uuid="])
    except GetoptError:
        usage()
        sys.exit(2)

    lock, unlock, name, uuid = False, False, "", ""
    for o, a in opts:
        if o in ("--lock",):
            lock = True
        elif o in ("--unlock",):
            unlock = True
        elif o in ("--name",):
            name = a
        elif o in ("--uuid",):
            uuid = a

    if (lock and unlock) or not (lock or unlock):
        print >>sys.stderr, "--lock or --unlock must be specified, and they are mutual-exclusive!"
        sys.exit(1)
    if not name or not uuid:
        print >>sys.stderr, "--name and --uuid must be specified!"
        sys.exit(1)

    output = ""
    try:
        if lock:
            cmd = "%s ha_set_dlm_lock %s %s" % (join(dirname(__file__), "do_rpc.py"), uuid, name)
        if unlock:
            cmd = "%s ha_release_dlm_lock %s %s" % (join(dirname(__file__), "do_rpc.py"), uuid, name)
        p = Popen(cmd.split(), stdout=PIPE)
        output = p.communicate()[0]
    except Exception, e:
        print >>sys.stderr, str(e)
        sys.exit(1)

    if "success" in output:
        print >>sys.stdout, output
        sys.exit(0)

    print >>sys.stderr, output
    sys.exit(1)

