#!/bin/bash
# Copyright (C) Oracle, 2008

# Prerequisite:
#   *) cfg file and images are in the same directory
# 
# $1: source cfg file path
# $2: target name

cfg_file_path="$1"
target_name="$2"

# Find/Replace text within a file                                              #
replace()
{
	SearchText=$1
	ReplaceText=$2
	File=$3
	if sed "s/$SearchText/$ReplaceText/g" "$File" > "$File.temporary" && sleep 1
	then
		if mv -f "$File.temporary" "$File" && sleep 1
		then
			return 0
		fi
	fi
	return 1
}

# Main entry

# source vm should be shutdown before clone
vmname=`grep "name =" "$cfg_file_path"`
vmname=${vmname#name =}
vmname=`echo $vmname | sed "s/\"//g"`
echo -e "source vm name: \"$vmname\""
if [[ "$vmname" != "" ]]; then
    xminfo=`xm list | grep "$vmname"`
    if [[ "$xminfo" != "" ]]; then
        echo -e "domU $vmname is currently running on Xen, please shutdown it before cloning."
        echo -e "$xminfo"
        exit 1
    fi
fi

src_dir=`dirname $cfg_file_path`
cfg_name=`basename $cfg_file_path`
#echo "$src_dir"
base_dir=${src_dir%/*}
#echo "$base_dir"
tgt_dir="$src_dir"/"$target_name"
if [[ "$src_dir" != "$base_dir" ]]; then
    tgt_dir="$base_dir"/"$target_name"
fi

if [[ -d "$tgt_dir" ]]; then
    echo -e "WARNING: Directory $tgt_dir existed!"
fi

echo -e "Creating the new image directory $tgt_dir"
if ! mkdir -pv --mode=775 "$tgt_dir"
then
    echo -e "Unable to create the directory $tgt_dir"
    exit 1
fi

echo -e "Copying cfg and complete image."
if ! cp -fv --sparse=never "$src_dir"/* "$tgt_dir"
then
    echo -e "Unable to copy the images from $src_dir to $tgt_dir"
    exit 1
fi

tgt_cfg="$tgt_dir"/vm.cfg
if [[ "$cfg_name" != "vm.cfg" ]]; then
    echo -e "Rename the cfg file from $tgt_dir/$cfg_name to $tgt_cfg."
    if ! mv "$tgt_dir"/"$cfg_name" "$tgt_cfg"
    then
        echo -e "Unable to rename $tgt_dir/$cfg_name to $tgt_cfg"
        exit 1
    fi
fi

# Replace disk path domU name, MAC, UUID in target config file

# domU name
echo -e "Correcting domU name from $vmname to $target_name."
if ! replace "name = \"$vmname\"" "name = \"$target_name\"" "$tgt_cfg"
then
    echo -e "Unable to correct domU name in $tgt_cfg"
    exit 1
fi

# MAC
old_MAC=`grep "vif =" $tgt_cfg`
old_MAC=${old_MAC#*[}
old_MAC=${old_MAC%*]}
old_MAC=`echo "$old_MAC" | cut -f2 -d\'`
old_MAC=`echo "$old_MAC" | cut -f2 -d\,`
old_MAC=${old_MAC# mac=*}
new_MAC=`python macgen.py`
echo -e "Correcting MAC from $old_MAC to $new_MAC."
if ! replace "mac=$old_MAC" "mac=$new_MAC" "$tgt_cfg"
then
    echo -e "Unable to correct MAC address in $tgt_cfg"
    exit 1
fi

# UUID

echo "Done"
