#!/bin/bash

# $1 - disk list file
# $2 - target image path

LC_ALL=C

ORIGINAL_IFS=$IFS
IFS=$'\n'

failed()
{
    echo "Failed"
    IFS=$ORIGINAL_IFS
    exit 1
}

usage()
{
    echo "Usage:"
    echo "$0 /path/to/vmdk/list/file /target/path"
    IFS=$ORIGINAL_IFS
    exit 1
}

if [[ $1 == "" || $2 == "" ]]; then
    usage
fi

raw_tmp_dir="/tmp/v2v_raw_tmp.$$"

mkdir "$raw_tmp_dir"
mkdir "$2"

for disk in `cat $1`;
do
    echo "$disk"
    dir=`dirname "$disk"`
    filename=`basename "$disk"`
    diskname=`echo ${filename%.*}`
    tgt_img_path="$2/$diskname.img"

    rm "$tgt_img_path"

    echo "$filename"
    magic=`head -n 1 $disk`
    if [[ $magic == "# Disk DescriptorFile" ]]; then
        # splitted vmdk
        subdisks=`grep "\.vmdk\"" "$disk" | sed 's/^.*\"\(.*\)\".*/\1/'`

        first=1
        for i in $subdisks;
        do
            echo "Converting $i"
            cd "$raw_tmp_dir" && qemu-img convert "$dir/$i" "$i.raw"
            if [[ $? -ne 0 ]]; then
                rm "$tgt_img_path"
                cd "$raw_tmp_dir" && rm *.raw
                failed
            fi

            echo "Joining $i"
            if [[ $first == 1 ]]; then
                first=0
                cd "$raw_tmp_dir" && mv "$i.raw" "$tgt_img_path"
            else
                cd "$raw_tmp_dir" && cat "$i.raw" >> "$tgt_img_path" && rm "$i.raw"
            fi
            if [[ $? -ne 0 ]]; then
                rm "$tgt_img_path"
                cd "$raw_tmp_dir" && rm *.raw
                failed
            fi
        done
    else
        # flattend vmdk
        echo "Converting $disk"
        qemu-img convert "$disk" "$tgt_img_path"
        if [[ $? -ne 0 ]]; then
            failed
        fi
    fi

done
echo "Done"
IFS=$ORIGINAL_IFS
exit 0
